#include <FL/fl_ask.h>

#include "ImpressionistView.h"
#include "ImpressionistModel.h"
#include "DotBrush.h"

//==========[ Constant Definitions ]===========================================

static const int BRUSH_POINTS = 0;
static const int BRUSH_LINES = 1;
static const int BRUSH_CIRCLES = 2;
static const int BRUSH_SCATTERED_POINTS = 3;
static const int BRUSH_SCATTERED_LINES = 4;
static const int BRUSH_SCATTERED_CIRCLES = 5;

//==========[ Static Variable Definitions ]====================================

Fl_Menu_Item ImpressionistView::menuItems[] = {
	{ "&File",		0, 0, 0, FL_SUBMENU },
		{ "&Load Image...",	FL_ALT + 'l', (Fl_Callback *)ImpressionistView::loadImageCB },
		{ "&Save Image...",	FL_ALT + 's', (Fl_Callback *)ImpressionistView::saveImageCB },
		{ "&Brushes...",	FL_ALT + 'b', (Fl_Callback *)ImpressionistView::brushesCB, 0, FL_MENU_DIVIDER },
		{ "&Exit",			FL_ALT + 'e', (Fl_Callback *)ImpressionistView::exitCB },
		{ 0 },

	{ "&Help",		0, 0, 0, FL_SUBMENU },
		{ "&About",	FL_ALT + 'a', (Fl_Callback *)ImpressionistView::aboutCB },
		{ 0 },

	{ 0 }
};

Fl_Menu_Item ImpressionistView::brushTypeMenu[] = {
  {"Points",			FL_ALT+'p', (Fl_Callback *)ImpressionistView::brushChoiceCB, (void *)BRUSH_POINTS},
  {"Lines",				FL_ALT+'l', (Fl_Callback *)ImpressionistView::brushChoiceCB, (void *)BRUSH_LINES},
  {"Circles",			FL_ALT+'c', (Fl_Callback *)ImpressionistView::brushChoiceCB, (void *)BRUSH_CIRCLES},
  {"Scattered Points",	FL_ALT+'q', (Fl_Callback *)ImpressionistView::brushChoiceCB, (void *)BRUSH_SCATTERED_POINTS},
  {"Scattered Lines",	FL_ALT+'m', (Fl_Callback *)ImpressionistView::brushChoiceCB, (void *)BRUSH_SCATTERED_LINES},
  {"Scattered Circles",	FL_ALT+'d', (Fl_Callback *)ImpressionistView::brushChoiceCB, (void *)BRUSH_SCATTERED_CIRCLES},
  {0}
};

//==========[ FLTK Callbacks ]=================================================

//----------[ loadImageCB ]----------------------------------------------------

void ImpressionistView::loadImageCB( Fl_Menu_* o, void* v ) {
	ImpressionistView*	view = (ImpressionistView*)(o->parent()->user_data());
	ImpressionistModel*	model = view->mModel;

	char* imageFileName = fl_file_chooser("Open File?", "*.bmp", "*.bmp" );
	if( imageFileName != NULL )
		model->loadImage( imageFileName );

	if( model->imageLoaded() ) {
		int width = model->getBitmapWidth();
		int height = model->getBitmapHeight();

		view->mMainWindow->resize(view->mMainWindow->x(), view->mMainWindow->y(), width*2, height+25);

		// display it on origView
		view->mOrigView->resizeWindow(width, height);	
		view->mOrigView->refresh();

		// refresh paint view as well
		view->mPaintView->resizeWindow(width, height);	
		view->mPaintView->refresh();
	}
}

//----------[ saveImageCB ]----------------------------------------------------

void ImpressionistView::saveImageCB( Fl_Menu_* o, void* v ) {
	ImpressionistView*	view = (ImpressionistView*)(o->parent()->user_data());
	ImpressionistModel*	model = view->mModel;

	char* imageFileName = fl_file_chooser("Save File?", "*.bmp", "save.bmp" );
	if( imageFileName != NULL )
		model->saveImage( imageFileName );
}

//----------[ brushesCB ]------------------------------------------------------

void ImpressionistView::brushesCB( Fl_Menu_* o, void* v ) {
	ImpressionistView*	view = (ImpressionistView*)(o->parent()->user_data());

	view->mBrushDialog->show();
}

//----------[ exitCB ]---------------------------------------------------------

void ImpressionistView::exitCB( Fl_Menu_* o, void* v ) {
	ImpressionistView*	view = (ImpressionistView*)(o->parent()->user_data());

	view->mMainWindow->hide();
	view->mBrushDialog->hide();
}

//----------[ windowCloseCB ]--------------------------------------------------

void ImpressionistView::windowCloseCB( Fl_Widget* o, void* v ) {
	ImpressionistView*	view = (ImpressionistView*)(o->user_data());

	view->mMainWindow->hide();
	view->mBrushDialog->hide();
}

//----------[ aboutCB ]--------------------------------------------------------

void ImpressionistView::aboutCB(Fl_Menu_* o, void* v) {
	fl_message("Impressionist FLTK version for CSE557, Winter 2001");
}

//----------[ sizeSlidesCB ]---------------------------------------------------

void ImpressionistView::sizeSlidesCB( Fl_Widget* o, void* v ) {
	ImpressionistView*	view = (ImpressionistView*)(o->user_data());
	ImpressionistModel*	model = view->mModel;

	model->setBrushSize( int(((Fl_Slider*)o)->value()) );
}

//----------[ brushChoiceCB ]--------------------------------------------------

void ImpressionistView::brushChoiceCB( Fl_Widget* o, void* v ) {
	ImpressionistView*	view = (ImpressionistView*)(o->user_data());
	ImpressionistModel* model = view->mModel;

	model->setCurrentBrush( view->mBrushes[(int)v] );
}

//==========[ ImpressionistView Methods ]======================================

//----------[ Default Constructor ]--------------------------------------------

ImpressionistView::ImpressionistView( ImpressionistModel* model ) {
	mModel = model;

	// create the actual brushes
	mBrushes = new AbstractBrush*[6];
	mBrushes[BRUSH_POINTS] = new DotBrush();
	// TODO: DotBrush is being used for all of the brushes.  Create the actual
	// brushes and replace the calls to new DotBrush() with the appropriate
	// constructor calls.  If you add any brushes besides these, make sure that
	// you update the brushTypeMenu and constant definitions at the top of the
	// file.
	mBrushes[BRUSH_LINES] = new DotBrush();
	mBrushes[BRUSH_CIRCLES] = new DotBrush();
	mBrushes[BRUSH_SCATTERED_POINTS] = new DotBrush();
	mBrushes[BRUSH_SCATTERED_LINES] = new DotBrush();
	mBrushes[BRUSH_SCATTERED_CIRCLES] = new DotBrush();

	// set the currently selected brush
	model->setCurrentBrush( mBrushes[BRUSH_POINTS] );

	// create the main window and add the GL viewports
	mMainWindow = new Fl_Window( 600, 300, "Impressionist" );
		// cache the pointer for use in callbacks
		mMainWindow->user_data( (void*)this );
		// set up the menu bar
		mMenubar = new Fl_Menu_Bar( 0, 0, 600, 25 );
		mMenubar->menu( menuItems );

		// create a group to hold the viewports
		Fl_Group* group = new Fl_Group( 0, 25, 600, 275 );
			// install the viewport for the impressionist painting
			mPaintView = new PaintView( model, 0, 25, 300, 275, "Impressionist View" );
			mPaintView->box( FL_DOWN_FRAME );

			// install the viewport for the original image
			mOrigView = new OriginalView( model, 300, 25, 300, 275, "Original View" );
			mOrigView->box( FL_DOWN_FRAME );
			mOrigView->deactivate();
		group->end();

		Fl_Group::current()->resizable( group );

		// set up the handler for window close events
		mMainWindow->callback( windowCloseCB );
		mMainWindow->when( FL_HIDE );
    mMainWindow->end();

	// create the brush dialog and associated controls
	mBrushDialog = new Fl_Window( 240, 80, "Brush Dialog" );
		// set up the brush size slider
		mBrushSizeSlider = new Fl_Value_Slider( 10, 45, 180, 20, "Size" );
		// cache the pointer for use in callbacks
		mBrushSizeSlider->user_data( (void*)this );
		mBrushSizeSlider->type( FL_HOR_NICE_SLIDER );
        mBrushSizeSlider->labelfont( FL_COURIER );
        mBrushSizeSlider->labelsize( 12 );
		mBrushSizeSlider->minimum( 0 );
		mBrushSizeSlider->maximum( 20 );
		mBrushSizeSlider->step( 1 );
		mBrushSizeSlider->value( model->getBrushSize() );
		mBrushSizeSlider->align( FL_ALIGN_RIGHT );
		mBrushSizeSlider->callback( sizeSlidesCB );

		// create the brush type pop-up menu
		mBrushTypeChoice = new Fl_Choice( 50,10,150,25,"&Brush" );
		// cache the pointer for use in callbacks
		mBrushTypeChoice->user_data( (void*)this );
		mBrushTypeChoice->menu( brushTypeMenu );
		mBrushTypeChoice->callback( brushChoiceCB );

		// TODO: add the various controls necessary for controlling brush
		// direction, line thickness, etc.
    mBrushDialog->end();	
}

//----------[ show ]-----------------------------------------------------------

void ImpressionistView::show() {
	mMainWindow->show();
	mPaintView->show();
	mOrigView->show();
}

//----------[ hide ]-----------------------------------------------------------

void ImpressionistView::hide() {
	mMainWindow->hide();
	mPaintView->hide();
	mOrigView->hide();
}